/**
 * @file util.h
 * @brief Prototipos, macros e tipos de dados de util.c
 * @date 15/02/2024
 * @author Wu Shin-Ting
 */

#ifndef UTIL_H_
#define UTIL_H_

#define GPIO_PIN(x)  ((1)<<(x))

typedef enum boolean_tag {
	OFF,     /**< falso/apaga/desativa/liga */
	ON      /**< verdadeiro/acende/ativa/fecha */
} tipo_booleano;

/**
 * @brief espera em multiplos de 2us
 * @param[in] multiplo de 2us
 */
void espera_2us (uint32_t multiplos);

/**
 * @brief espera em decrementos de valores ate 0
 * @param[in] valor de decremento
 */
void espera (uint32_t valor);

/**
 * @brief Converte um valor inteiro para uma string de caracteres ASCII
 * @param[out] s string
 * @param[in] bin valor inteiro
 * @param[in] n quantidade de digitos
 * 
 * @note https://community.arm.com/support-forums/f/keil-forum/17118/conversion-of-integer-to-ascii-for-display
 */
void UlToStr(char *s, uint32_t bin, uint8_t n);

/**
 * @brief Converte segundos em dias
 * @param[in] n segundos
 * @param[out] dias
 * @param[out] horas
 * @param[out] minutos
 * @param[out] segundos
 * 
 * @note https://www.geeksforgeeks.org/converting-seconds-into-days-hours-minutes-and-seconds/
 */
void ConvertSectoDay(uint32_t n, uint32_t *dias, uint32_t *horas, uint32_t *minutos, uint32_t *segundos);


/**
 * @brief Converte segundos em dias no formato HH:MM:SS
 * @param[in] n segundos
 * @return string no formato HH:MM:SS
 * 
 */
char *ConvertSectoDayString (uint32_t seconds, char *string);

/**
 * @brief Converte dias em segundos
 * @param[in] dias
 * @param[in] horas
 * @param[in] minutos
 * @param[in] segundos
 * @param[out] n segundos
 * 
 */
void ConvertDaytoSec(uint32_t days, uint32_t hours, uint32_t minutes, uint32_t seconds, uint32_t *n);

/**
 * @brief Converte string de digitos decimais em valor inteiro sem sinal
 * @param[in] str string de digitos decimais sem sinal
 * @param[out] 
 * @return codigo de erro (0:Ok; 2: digito invalido; 3: mais de 10 digitos) 
 */
uint8_t ConvertDecStringtoUl32 (char *str, uint32_t *valor);

/**
 * @brief Converte um valor numa base (ate hex) para uma string de digitos da base 
 * @param[in] n valor
 * @param[out] string
 * @param[in] b base
 * @return ponteiro da string 
 */
char *ConvertI32toString (int32_t n, char *string, uint8_t b);

/**
 * @brief Inverter uma sequencia de elementos de uma string
 * @param[in] string
 * @param[in] len quantidade de elementos na string
 * @return ponteiro a string invertida
 */
void reverse(char* str, uint8_t len);

/**
 * @brief Converte um valor inteiro sem sinal para uma string com quantidade de digitos definida
 * @param[in] x valor
 * @param[out] str endereco da string
 * @param[in] d qtde de digitos, completando os digitos omitido com '0'
 * @return qtde de digitos na string
 */
int intToStr(uint32_t x, char str[], uint8_t d);

/**
 * @brief Converte um valor em ponto flutuante, sem sinal, para uma string com a 
 * qte de digitos decimais especificada
 * @param[in] n valor inteiro
 * @param[out] res string
 * @param[in] afterpoint qtde dde digitos decimais
 */
void ftoa(float n, char* res, uint8_t afterpoint);
#endif /* UTIL_H_ */
