/**
 * @file util.c
 * @brief Definicao das funcoes uteis do projeto
 * @date Jan 11, 2023
 * @author Wu Shin-Ting   
 */
//Inclusao dos tipos de dados padronizados pelo ISO C99
#include "stdint.h"
#include "string.h"
#include "stdlib.h"
#include "math.h"

void espera_2us (uint32_t multiplos)
{
		__asm__ (
			// substitua pelo seu codigo do roteiro 3
			"NOP \n\t"
		);
}

void espera (uint32_t valor)
{
	while (valor) valor--;
}

void UlToStr(char *s, uint32_t bin, uint8_t n)
{
    s += n;
    *s = '\0';

    while (n--)
    {
        *--s = (bin % 10) + '0';
        bin /= 10;
    }
}

void ConvertSectoDay(uint32_t n, uint32_t *days, uint32_t *hours, uint32_t *minutes, uint32_t *seconds)
{
    *days = n / (24 * 3600);
 
    n = n % (24 * 3600);
    *hours = n / 3600;
 
    n %= 3600;
    *minutes = n / 60 ;
 
    n %= 60;
    *seconds = n;     
}

char *ConvertSectoDayString (uint32_t seconds, char *string)
{
	uint32_t dd, hh, mm, ss;

	ConvertSectoDay (seconds, &dd, &hh, &mm, &ss);

	string[2] = string[5] = ':';    // inserir os espacadores
	string[8] = '\0';				// terminador

	if (hh > 23) {
		//!< horario invalido: FF:FF:FF
		string[0] = string[1] = string[3] = string[4] = string[6] = string[7] = 'F';
	}

	string[0] = (hh < 10)? '0': (hh/10)+'0';
	string[1] = hh%10+'0';

	string[3] = (mm < 10)? '0': (mm/10)+'0';
	string[4] = mm%10+'0';

	string[6] = (ss < 10)? '0': (ss/10)+'0';
	string[7] = ss%10+'0';

	return string;
}

void ConvertDaytoSec(uint32_t days, uint32_t hours, uint32_t minutes, uint32_t seconds, uint32_t *n)
{
	*n = days*86400+hours*3600+minutes*60+seconds;
}

uint8_t ConvertDecStringtoUl32 (char *str, uint32_t *valor) {

	uint8_t i;
	uint8_t digitos = strlen(str);
	*valor = 0;
	uint32_t pot = 1;
	if (digitos > 10) return 3;
	for (i=0; i<digitos; i++) {
		uint8_t aux = digitos-i-1;			
		if(str[aux] < 0x30 || str[aux] > 0x39) return 2;
		
		*valor += (str[aux]-0x30)*pot;
		pot *= 10;
	}
	
	return 0;
}

char *ConvertI32toString (int32_t n, char *string, uint8_t b)
{
	char sinal;
	unsigned short i, j, a;
	unsigned int m;
	
	if (n == 0) {
		///< Tratamento do caso especial
		string[0] = '0';
		string[1] = '\0';
		return string;
	}
	
	if (n < 0) sinal = 1;
	else sinal = 0;
	
	/*!
	 * Converter para codigos ASCII, digito por digito, de menos 
	 * significativo para o mais
	 */ 
	i = 0;
	m = abs(n);
	
	while (m != 0) {
		a = m%b;
		string[i+sinal] = (a>9)?(a-10)+'A' : a+'0';
		i++;
		m = m/b;
	}
	string[i+sinal] = '\0';			///< Incluir terminador
	
	///< Reverter a sequencia
	j = strlen(string)-1;
	i = sinal;
			
	while (j > i) {
		char tmp = string[i];
		string[i] = string[j];
		string[j] = tmp;
		i++;
		j--;
	}
	
	///< Adicionar o sinal negativo caso necessario
	if (sinal) string[0] = '-';
	
	return string;
}

void reverse(char* str, uint8_t len)
{
    int i = 0, j = len - 1, temp;
    while (i < j) {
        temp = str[i];
        str[i] = str[j];
        str[j] = temp;
        i++;
        j--;
    }
}

int intToStr(uint32_t x, char str[], uint8_t d)
{
    int i = 0;
    do {
    	str[i++] = (x % 10) + '0';
    	x = x / 10;
    }
    while (x);

    while (i < d) str[i++] = '0';
    reverse(str, i);
    str[i] = '\0';
    return i;
}

//Converte um float em uma string com afterpoint digitos depois do ponto
void ftoa(float n, char* res, uint8_t afterpoint){ 
	
	// Extrai a parte inteira
	int ipart = (int)n;
	  
	// Extrai a parte em ponto flutuante
	float fpart = n - (float)ipart;
	
	// Converte a parte inteira para string
	int i = intToStr(ipart, res, 0);
	  
	if (afterpoint != 0) {
	   res[i] = '.'; // Adiciona o ponto
	   fpart = fpart * pow(10, afterpoint); //Desloca a parte significativa depois do ponto

	   if (fpart - (int)fpart >= 0.5) fpart += 1.0;    // Arredondamento e nao truncamento
	   intToStr((int)fpart, res + i + 1, afterpoint);
	}

	return;
}


